// src/index.ts
import { dirname } from "node:path";
import { fileURLToPath } from "node:url";
import eslint from "@eslint/js";
import eslintPluginImport from "eslint-plugin-import";
import eslintPluginN from "eslint-plugin-n";
import tseslint from "typescript-eslint";

// src/types.ts
import "typescript-eslint";

// src/index.ts
var JS_EXTENSIONS = [".js", ".jsx", ".mjs", ".cjs"];
var TS_EXTENSIONS = [".ts", ".tsx", ".mts", ".cts"];
var EXTENSIONS = [...TS_EXTENSIONS, ...JS_EXTENSIONS];
var RULES = {
  base: {
    // ESLINT default rules
    "arrow-body-style": "warn",
    camelcase: "warn",
    curly: ["warn", "all"],
    "default-case": "warn",
    "default-case-last": "error",
    "default-param-last": "error",
    "dot-notation": "warn",
    eqeqeq: [
      "warn",
      "always",
      {
        null: "ignore"
      }
    ],
    "guard-for-in": "warn",
    "no-bitwise": "warn",
    "no-console": "warn",
    "no-constructor-return": "error",
    "no-else-return": "warn",
    "no-iterator": "error",
    "no-lone-blocks": "warn",
    "no-lonely-if": "error",
    "no-loop-func": "error",
    "no-param-reassign": ["warn", { props: true }],
    "no-plusplus": ["error", { allowForLoopAfterthoughts: true }],
    "no-promise-executor-return": "error",
    "no-proto": "error",
    "no-return-assign": "error",
    "no-sequences": "error",
    "no-undef": "warn",
    "no-unmodified-loop-condition": "error",
    "no-useless-assignment": "error",
    "object-shorthand": "warn",
    "prefer-const": "warn",
    "prefer-promise-reject-errors": "error",
    "sort-imports": [
      "warn",
      {
        ignoreCase: true,
        ignoreDeclarationSort: true
      }
    ],
    // eslint-plugin-import
    "import/no-cycle": "error"
  },
  node: {
    // eslint-plugin-n
    "n/prefer-node-protocol": "warn"
  },
  js: {
    // Only JS
    "no-duplicate-imports": "error",
    // Conflicts with typescript
    "consistent-return": "warn",
    "no-shadow": "error",
    "no-unused-vars": [
      "warn",
      {
        args: "after-used",
        argsIgnorePattern: "^_",
        caughtErrors: "all",
        caughtErrorsIgnorePattern: "^_",
        destructuredArrayIgnorePattern: "^_",
        varsIgnorePattern: "^_"
      }
    ],
    "prefer-destructuring": "warn",
    "require-await": "warn"
  },
  ts: {
    // typescript-eslint
    "@typescript-eslint/array-type": ["warn", { default: "array-simple" }],
    "@typescript-eslint/prefer-nullish-coalescing": [
      "warn",
      {
        ignoreConditionalTests: true,
        ignoreMixedLogicalExpressions: true,
        ignorePrimitives: { string: true, boolean: true }
      }
    ],
    "@typescript-eslint/consistent-type-definitions": "off",
    "@typescript-eslint/no-extraneous-class": ["error", { allowWithDecorator: true }],
    "@typescript-eslint/no-inferrable-types": [
      "warn",
      {
        ignoreParameters: true,
        ignoreProperties: true
      }
    ],
    "@typescript-eslint/no-invalid-void-type": "off",
    "@typescript-eslint/no-non-null-assertion": "warn",
    // BUG: This rules sometimes removes types that convert to any when the ts plugin fails
    "@typescript-eslint/no-unnecessary-type-arguments": "off",
    "@typescript-eslint/prefer-string-starts-ends-with": [
      "warn",
      {
        allowSingleElementEquality: "always"
      }
    ],
    "@typescript-eslint/restrict-template-expressions": [
      "warn",
      {
        allowNumber: true
      }
    ],
    "@typescript-eslint/switch-exhaustiveness-check": ["warn"],
    "@typescript-eslint/unified-signatures": "off",
    // Rules that conflicts with default eslint
    "consistent-return": "off",
    "@typescript-eslint/consistent-return": "warn",
    "no-shadow": "off",
    "@typescript-eslint/no-shadow": "error",
    "no-unused-vars": "off",
    "@typescript-eslint/no-unused-vars": [
      "warn",
      {
        args: "after-used",
        argsIgnorePattern: "^_",
        caughtErrors: "all",
        caughtErrorsIgnorePattern: "^_",
        destructuredArrayIgnorePattern: "^_",
        varsIgnorePattern: "^_"
      }
    ],
    "prefer-destructuring": "off",
    "@typescript-eslint/prefer-destructuring": [
      "warn",
      { VariableDeclarator: { object: true } }
    ],
    "require-await": "off",
    "@typescript-eslint/require-await": "warn",
    // Rules that does not work on ts
    "no-duplicate-imports": "off",
    "import/consistent-type-specifier-style": ["warn", "prefer-inline"],
    "import/no-duplicates": [
      "warn",
      {
        considerQueryString: true,
        "prefer-inline": true
      }
    ]
  },
  cjs: {
    "@typescript-eslint/no-require-imports": "off"
  }
};
var config = {
  ts: (options) => createRules({
    options: options ?? null,
    extraConfig: {
      name: "@rhangai/esling-config-typescript/ts-node",
      plugins: {
        n: eslintPluginN
      },
      rules: RULES.node
    }
  }),
  tsWeb: (options) => createRules({
    options: options ?? null
  })
};
var src_default = config;
function createRules({ options, extraConfig }) {
  const rootDir = resolveRootDir(options);
  const devFiles = options?.devFiles ?? [];
  const { restrictExtraConfig, restrictRules } = importRestrictRules(options);
  const ignores = options?.ignores;
  const globals = options?.globals;
  const eslintConfigs = [
    ignores ? { ignores } : null,
    eslint.configs.recommended,
    ...tseslint.configs.strictTypeChecked,
    ...tseslint.configs.stylisticTypeChecked,
    {
      name: "@rhangai/esling-config-typescript/ts",
      plugins: {
        import: eslintPluginImport
      },
      languageOptions: {
        parserOptions: {
          project: true,
          tsconfigRootDir: rootDir,
          ...options?.parserOptions
        },
        ...globals ? { globals } : {}
      },
      rules: {
        ...RULES.base,
        ...RULES.ts,
        ...restrictRules,
        "import/no-extraneous-dependencies": [
          "error",
          {
            packageDir: options?.monorepoPackageJson ? rootDir : void 0,
            devDependencies: [
              "eslint.config.*",
              "**/*.test.*",
              "**/*.spec.*",
              "**/spec/**/*",
              "**/test/**/*",
              "**/__tests__/**/*",
              ...devFiles
            ]
          }
        ],
        "import/order": [
          "error",
          importOrderOptions(options?.priorityPackages, options?.localPackages)
        ]
      },
      settings: {
        "import/extensions": EXTENSIONS,
        "import/external-module-folders": ["node_modules", "node_modules/@types"],
        "import/parsers": {
          "@typescript-eslint/parser": TS_EXTENSIONS
        },
        "import/resolver": {
          node: {
            extensions: EXTENSIONS
          }
        },
        "import/internal-regex": options?.internalPackagesRegex
      }
    },
    ...restrictExtraConfig,
    extraConfig,
    {
      ...tseslint.configs.disableTypeChecked,
      name: "@rhangai/esling-config-typescript/ts-js",
      files: [`**/*.{${JS_EXTENSIONS.map((e) => e.substring(1)).join(",")}}`],
      rules: {
        ...tseslint.configs.disableTypeChecked.rules,
        ...RULES.js
      }
    },
    {
      ...tseslint.configs.disableTypeChecked,
      name: "@rhangai/esling-config-typescript/ts-cjs",
      files: [`**/*.cjs`, ...options?.cjsFiles ?? []],
      rules: {
        ...RULES.cjs
      },
      languageOptions: {
        globals: {
          module: "readonly",
          require: "readonly",
          __dirname: "readonly",
          __filename: "readonly"
        }
      }
    }
  ];
  return eslintConfigs.filter(Boolean);
}
function resolveRootDir(options) {
  if (!options) {
    return void 0;
  }
  const { meta } = options;
  if (!meta) {
    return void 0;
  }
  if (meta.dirname) {
    return meta.dirname;
  }
  if (meta.url) {
    return dirname(fileURLToPath(meta.url));
  }
  return void 0;
}
function importOrderOptions(packages, localPackages) {
  const pathGroups = [];
  if (packages) {
    for (const packageName of packages) {
      pathGroups.push({
        pattern: packageName,
        group: "external",
        position: "before"
      });
    }
  }
  if (localPackages) {
    for (const packageName of localPackages) {
      pathGroups.push({
        pattern: packageName,
        group: "internal",
        position: "after"
      });
    }
  }
  return {
    groups: [
      //
      "builtin",
      "external",
      "internal",
      "parent",
      "sibling",
      "index"
    ],
    pathGroups,
    pathGroupsExcludedImportTypes: ["builtin"],
    alphabetize: {
      order: "asc",
      caseInsensitive: true
    }
  };
}
function importRestrictRules(options) {
  const restrictExtraConfig = [];
  const restrictRules = {};
  const restrict = options?.restrict;
  if (!restrict) {
    return {
      restrictRules,
      restrictExtraConfig
    };
  }
  const restrictRulePatterns = [];
  const restrictRulePaths = [];
  for (const restriction of restrict) {
    const patterns = restrictionPatternMap(restriction);
    const paths = restrictionPathMap(restriction);
    if (!restriction.files || restriction.files.length <= 0) {
      if (patterns) {
        restrictRulePatterns.push(...patterns);
      }
      if (paths) {
        restrictRulePaths.push(...paths);
      }
      continue;
    }
    if (!patterns && !paths) {
      continue;
    }
    const ruleConfig = {};
    if (patterns) {
      ruleConfig.patterns = patterns;
    }
    if (paths) {
      ruleConfig.paths = paths;
    }
    restrictExtraConfig.push({
      files: Array.isArray(restriction.files) ? restriction.files : [restriction.files],
      rules: {
        "no-restricted-imports": ["error", ruleConfig]
      }
    });
  }
  if (restrictRulePatterns.length > 0 || restrictRulePaths.length > 0) {
    restrictRules["no-restricted-imports"] = [
      "error",
      {
        patterns: restrictRulePatterns,
        paths: restrictRulePaths
      }
    ];
  }
  return {
    restrictRules,
    restrictExtraConfig
  };
}
function restrictionPatternMap(restriction) {
  if (!restriction.patterns) {
    return null;
  }
  const patterns = [];
  for (const [name, group] of Object.entries(restriction.patterns)) {
    if (Array.isArray(group)) {
      if (group.length <= 0) {
        continue;
      }
      patterns.push({
        group,
        message: `N\xC3O pode importar de "${name}"`
      });
    } else if (group) {
      patterns.push({
        group: [group],
        message: `N\xC3O pode importar de "${name}"`
      });
    }
  }
  if (patterns.length <= 0) {
    return null;
  }
  return patterns;
}
function restrictionPathMap(restriction) {
  if (!restriction.paths) {
    return null;
  }
  const paths = [];
  for (const [name, message] of Object.entries(restriction.paths)) {
    paths.push({
      name,
      message: message || `N\xC3O pode importar de "${name}"`
    });
  }
  if (paths.length <= 0) {
    return null;
  }
  return paths;
}
export {
  src_default as default
};
